import logging
import functools
from pathlib import Path
import jinja2
from jinja2 import Environment, PackageLoader

from . import config

__version__ = '0.0.1'

APP_DIR = Path(__file__).parent
INPUT_DIR = Path(config.INPUT_PATH)
CONTENT_DIR = INPUT_DIR / config.CONTENT_DIR_NAME
OUTPUT_DIR = Path(config.OUTPUT_PATH)
IGNORE_ASSETS = {'__init__.py', '__pycache__'}

logging.basicConfig(level=config.LOG_LEVEL, format=config.LOG_FORMAT)

jinja = Environment(
    loader=PackageLoader('input'),
    autoescape=jinja2.select_autoescape(),
)

def filter(f):
    jinja.filters[f.__name__] = f
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        return f(*args, **kwargs)
    return wrapper

from . import filters



import hashlib
import jinja2

import sssimp


@sssimp.filter
@jinja2.pass_context
def a(context, value: str):
    """Makes a path absolute no matter where the file is"""
    value = value.removeprefix('/')
    l1 = set(context['path'].parents)
    l2 = set(sssimp.OUTPUT_DIR.parents)
    l2.add(sssimp.OUTPUT_DIR)
    diff = l1.difference(l2)
    for _ in diff:
        value = '../' + value
    return value



import importlib
import logging
from pathlib import Path
import shutil

import sssimp
from sssimp import config
from sssimp.utils import mkdir, path_strip

def run_generators():
    for file in (sssimp.APP_DIR / 'generators').glob('**/*.py'):
        module_str = path_strip(file).replace('/', '.')[:-len('.py')]
        module = importlib.import_module(f'.{module_str}', package='sssimp')
        logging.info(f'Running {module.__name__}')
        module.main()

def is_ignored(path: Path):
    for ignore_path in sssimp.IGNORE_ASSETS:
        if ignore_path == str(path):
            return ignore_path
        if Path(ignore_path).is_dir() and ignore_path in {str(x) for x in path.parents}:
            return ignore_path

def copy_assets():
    logging.debug(f'Ignore list: {sssimp.IGNORE_ASSETS}')
    for file in sssimp.CONTENT_DIR.glob('**/*'):
        if not file.is_file():
            continue
        relative_path = path_strip(file, sssimp.CONTENT_DIR)
        if (match := is_ignored(file)):
            logging.info(
                f'Ignoring asset {relative_path} because it matches'
                f' "{match}" from ignore list'
            )
            continue
        out_file = sssimp.OUTPUT_DIR / relative_path
        mkdir(out_file)
        logging.info(f'Copying raw asset {file} -> {out_file}')
        shutil.copyfile(file, out_file)


if __name__ == '__main__':
    if config.CLEAN_OUTPUT and sssimp.OUTPUT_DIR.exists():
        shutil.rmtree(sssimp.OUTPUT_DIR)
        logging.info(f'Deleted {sssimp.OUTPUT_DIR}')
    logging.info('Running generators')
    run_generators()
    logging.info('Copying raw assets')
    copy_assets()
